import ctypes              # Python ctypes - https://docs.python.org/3/library/ctypes.html
import pathlib             # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import platform            # Python Platform - https://docs.python.org/3/library/platform.html
import sys                 # Python Sys - https://docs.python.org/3/library/sys.html

import wx.adv              # wxPython Advanced - https://docs.wxpython.org/wx.adv.1moduleindex.html
import wx.html             # wxPython HTML - https://docs.wxpython.org/wx.html.1moduleindex.html

from commit.includes.commitapp import CommitApp     # local - override app (for Mac file handling)
from commit import constants                        # local - app constants
from commit.config import AppConfig                 # local - app config (preferences)
from commit.includes.appupdater import AppUpdater   # local - app updater mechanism
from commit.includes.recentlist import RecentList   # local - recent list maintainer
from commit.ui.mainwin import MainWin               # local - main window


# Launch the application
if __name__ == '__main__':
    # prevent pointless logging; comment this out to deal with bugs on Py2Exe :-)
    sys.stderr = sys.stdout

    # enable high-dpi for Windows
    if platform.system() == "Windows":
        try:
            ctypes.windll.shcore.SetProcessDpiAwareness(True)
        except:  # NOQA; we really want to just continue if this fails for any reason
            pass

    app = CommitApp(
        useBestVisual=True
    )

    # set important app and vendor attributes from constant values
    app.SetAppName(constants.app_name)
    app.SetAppDisplayName(constants.app_display_name)
    app.SetVendorName(constants.vendor_name)
    app.SetVendorDisplayName(constants.vendor_display_name)

    # save our constants to the app object for use elsewhere
    app.constants = constants

    # init translations
    _ = app.constants.translation.gettext

    # set/update the constants that have to be done at runtime

    # modify the product code depending on OS
    if app.constants.platform == "Windows":
        this_replace_string = "A"
    elif app.constants.platform == "Darwin":  # MacOS
        this_replace_string = "B"
    elif app.constants.platform == "Linux":
        this_replace_string = "C"
    elif app.constants.platform == "Haiku":  # wishful thinking (not working yet)
        this_replace_string = "D"
    else:  # in case it gets ported elsewhere; can add more OSs later
        this_replace_string = "X"
    app.constants.vendor_product_code = app.constants.vendor_product_code.replace("$", this_replace_string)

    # set up the app icon
    app.constants.icon = wx.IconBundle()
    app.constants.icon.AddIcon("commit/assets/icons/appicon16.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon24.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon32.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon48.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon64.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon96.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon128.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon256.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon512.png", wx.BITMAP_TYPE_PNG)
    app.constants.icon.AddIcon("commit/assets/icons/appicon1024.png", wx.BITMAP_TYPE_PNG)

    # app config (preferences) object
    app.config = AppConfig()

    # some product info to the console
    print(constants.app_display_name + ": " + constants.app_tagline)
    print(constants.vendor_display_name + " - " + app.constants.vendor_product_code)

    # detect command line arguments
    if len(sys.argv) > 1:
        app.file_on_open = pathlib.Path(sys.argv[1])

    # init the recent files list
    app.recents = RecentList()

    # create the main window (will show automatically)
    main_window = MainWin(None, app.file_on_open)

    # init the app updater
    app.updater = AppUpdater(main_window)

    # init the print system
    app.printer = wx.html.HtmlEasyPrinting(
        app.constants.app_display_name,
        main_window
    )

    # show the splash screen, unless suppressed by settings
    if app.config.ReadBool("startup/show_splash"):
        splash = wx.adv.SplashScreen(
            wx.Bitmap(
                "commit/assets/commit-splash.png",
                wx.BITMAP_TYPE_PNG
            ),
            wx.adv.SPLASH_CENTER_ON_PARENT | wx.adv.SPLASH_TIMEOUT,
            2000,
            main_window
        )

    # apploopy
    app.MainLoop()
