import re               # Python Regex - https://docs.python.org/3/library/re.html
import time             # Python Time - https://docs.python.org/3/library/time.html
import urllib.request   # Python URLLib - https://docs.python.org/3/library/urllib.request.html
import webbrowser       # Python WebBrowser - https://docs.python.org/3/library/webbrowser.html

import wx               # wxPython - https://wxpython.org/


class AppUpdater:
    def __init__(self, window):
        # get app obj
        self.app = wx.GetApp()
        self._ = self.app.constants.translation.gettext

        # get the window
        self.window = window

        # get the interval setting
        interval = self.app.config.ReadInt("updates/interval", 2)
        self.last_check = self.app.config.ReadInt("updates/last_check", 0)

        # if interval isn't "never"
        if interval > 0:
            # right now
            now = int(time.time())

            # auto-check interval mapping
            intervals = [
                0,
                24 * 60 * 60,
                7 * 24 * 60 * 60,
                30 * 24 * 60 * 60
            ]

            # are we past the interval? if so, check!
            if now >= self.last_check + intervals[interval]:
                self.do_update_check(auto_check=True)

    def do_update_check(self, calling_window=None, auto_check=False):
        # now
        now = int(time.time())

        # throttle to one check per 10 seconds
        if now < self.last_check + 10:
            return

        # get latest version data
        latest_data = urllib.request.urlopen(self.app.constants.app_update_url)

        # compare!
        need_update = self.compare_version_numbers(
            self.app.constants.app_version,
            latest_data.read().decode("UTF-8")
        )

        # save the check time back to the obj and the setting
        self.last_check = now
        self.app.config.WriteInt("updates/last_check", now)

        if need_update:
            update_dialog = wx.MessageDialog(
                None,
                self._("An update to Commit is available. Would you like to close Commit, go the website, and download the latest version?"),
                self._("Software Update Available"),
                wx.YES_NO | wx.YES_DEFAULT | wx.ICON_INFORMATION
            )

            # ask?
            confirm_result = update_dialog.ShowModal()

            # handle the answers
            if confirm_result == wx.ID_YES:
                webbrowser.open(wx.GetApp().constants.app_website)

                # close the dialog that called us (if any)
                if calling_window is not None:
                    calling_window.Destroy()

                # then close the main window/app
                self.window.Destroy()
        elif not auto_check:
            wx.MessageBox(
                self._("You already have the latest version of Commit."),
                self._("No Updates Available"),
                wx.OK | wx.ICON_INFORMATION
            )

    # compare version numbers; return TRUE if check_version > app_version
    # expected version number formats are 0.0.0 [stable] or 0.0.0X0 [where the X can be a, b, rc]
    @staticmethod
    def compare_version_numbers(app_version, check_version):
        app_version_split = app_version.split(".")
        check_version_split = check_version.split(".")

        for idx, num in enumerate(app_version_split):
            if idx < 2:
                # first two positions, simple number comparison
                # only continue checking if they exactly match
                if num < check_version_split[idx]:
                    return True
                elif num > check_version_split[idx]:
                    return False
            else:
                # split it up
                regex_obj = re.compile(r"^([0-9]+)([a-z]*)([0-9]*)$")
                current_match = regex_obj.search(num)
                latest_match = regex_obj.search(check_version_split[idx])

                # compare the first position (which should always be there)
                if current_match.group(1) < latest_match.group(1):
                    return True
                elif current_match.group(1) > latest_match.group(1):
                    return False

                # check if we even have any a/b/rc values; if not we're done!
                if (
                    current_match.group(2) == "" and
                    latest_match.group(2) == ""
                ):
                    return False

                # now let's check the situation where one is a release and the other is a/b/rc
                if (
                    current_match.group(2) != "" and
                    latest_match.group(2) == ""  # we're an a/b/rc, but the release is out
                ):
                    return True
                elif (
                    current_match.group(2) == "" and
                    latest_match.group(2) != ""  # we're a release, but latest is a/b/rc
                ):
                    return False

                # now we really need to compare the values
                type_map = [
                    "a",
                    "b",
                    "rc",
                    None
                ]

                # compare type (if we are alpha, but latest is beta, we need update...etc.)
                if type_map.index(current_match.group(2)) < type_map.index(latest_match.group(2)):
                    return True
                elif type_map.index(current_match.group(2)) > type_map.index(latest_match.group(2)):
                    return False

                # now compare the numbers
                if current_match.group(3) < latest_match.group(3):
                    return True
                else:
                    return False

        return False
