import pathlib          # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import re               # Python Regex - https://docs.python.org/3/library/re.html

import pypandoc         # pypandoc - https://github.com/JessicaTegner/pypandoc
import wx               # wxPython - https://wxpython.org/


class FileConvert:
    def __init__(self):
        # get app obj
        self.app = wx.GetApp()
        self._ = self.app.constants.translation.gettext

        # these are the formats we want to support (will be checked against what Pandoc can do)
        self.supported_formats = {
            "html": [
                self._("HyperText Markup Language"),
                ".html"
            ],
            "markdown_strict": [
                self._("Markdown Text"),
                ".md"
            ],
            "docx": [
                self._("Microsoft Word"),
                ".docx"
            ],
            "odt": [
                self._("OpenDocument Text"),
                ".odt"
            ],
            "rtf": [
                self._("Rich Text Format"),
                ".rtf"
            ],
            "txt": [
                self._("Plain Text"),
                ".txt"
            ]
        }

        # usual arguments for pandoc conversions
        self.args = ["-s", "--wrap=none"]

    def pandoc_check(self):
        try:
            pypandoc.get_pandoc_path()
            return True
        except OSError:
            # build dialog
            pandoc_dialog = wx.MessageDialog(
                None,
                self._("Import and export features require the installation of Pandoc, an open source document conversion tool. Would you like to automatically download and install Pandoc?"),
                self._("Install Pandoc?"),
                wx.YES_NO | wx.YES_DEFAULT | wx.ICON_INFORMATION
            )

            # ask?
            confirm_result = pandoc_dialog.ShowModal()

            # handle the answers
            if confirm_result == wx.ID_NO:
                return False
            elif confirm_result == wx.ID_YES:
                try:
                    pypandoc.pandoc_download.download_pandoc(
                        delete_installer=True,
                        download_folder=str(self.app.config.path)
                    )
                except OSError:
                    return False

                return True
            else:
                return False

    # do an import
    def do_import(self, window):
        pandoc_from_formats, pandoc_to_formats = pypandoc.get_pandoc_formats()
        ext_to_pandoc_map = {}
        dialog_format_string = ""
        all_formats_string = ""

        for this_format in self.supported_formats:
            if this_format == "txt" or this_format in pandoc_from_formats:
                # gather info
                this_type = self.supported_formats[this_format][0]
                this_ext = self.supported_formats[this_format][1]
                formatted_ext = "*" + this_ext

                # add to the ext/pandoc mapping
                ext_to_pandoc_map[this_ext] = this_format

                # add to the dialog format string
                dialog_format_string += this_type + " (" + formatted_ext + ")|" + formatted_ext + "|"

                # add to the all formats string
                if all_formats_string != "":
                    all_formats_string += ";"
                all_formats_string += formatted_ext

        # finalize the format string
        # note that we trim off the last | char that's dangling on the end
        dialog_format_string = "All supported formats... (" + all_formats_string + ")|" + all_formats_string + "|" + dialog_format_string[0:-1]

        # build the import dialog
        import_dialog = wx.FileDialog(
            window,
            self._("Import from file..."),
            wildcard=dialog_format_string,
            defaultDir=str(window.file.find_default_dir()),
            style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
        )
        if import_dialog.ShowModal() == wx.ID_CANCEL:
            return

        # get file info via pathlib
        file = pathlib.Path(import_dialog.GetPath())

        # use pandoc to open up all other supported document types
        try:
            # txt needs special handling
            if file.suffix == ".txt":
                output = pypandoc.convert_text(
                    self.txt_to_md(file.read_text()),
                    format="markdown",
                    to="markdown",
                    outputfile=None,  # returning as a string in this case
                    extra_args=self.args
                )
            else:
                output = pypandoc.convert_file(
                    file,
                    format=ext_to_pandoc_map[file.suffix],
                    to="markdown",
                    extra_args=self.args
                )
        except Exception:  # NOQA
            wx.MessageBox(
                self._("Unable to import the specified file."),
                self._("Error"),
                wx.OK | wx.ICON_ERROR
            )
            return

        # normalize the line breaks!
        output = self.normalize_line_breaks(output)

        # send the imported content to the editor
        window.file.open(None, is_import=True, raw_text=output)

    # method for exporting files to supported file formats
    def do_export(self, window, content):
        pandoc_from_formats, pandoc_to_formats = pypandoc.get_pandoc_formats()
        ext_to_pandoc_map = {}
        dialog_format_string = ""

        # Our native format is not QUITE valid Markdown, which can cause problems.
        # Specifically, we treat single line breaks as line breaks, while Markdown treats them
        # as spaces. We need to add two spaces to EOL to make it work right on export.
        content = re.sub(r"(?<!\n)(\n)(?!\n)", r"  \n", content)

        for this_format in self.supported_formats:
            if this_format == "txt" or this_format in pandoc_to_formats:
                # gather info
                this_type = self.supported_formats[this_format][0]
                this_ext = self.supported_formats[this_format][1]
                formatted_ext = "*" + this_ext

                # add to the ext/pandoc mapping
                ext_to_pandoc_map[this_ext] = this_format

                # add to the dialog format string
                dialog_format_string += this_type + " (" + formatted_ext + ")|" + formatted_ext + "|"

        # finalize the format string
        # note that we trim off the last | char that's dangling on the end
        dialog_format_string = dialog_format_string[0:-1]

        # build the export dialog
        export_dialog = wx.FileDialog(
            window,
            self._("Export to file..."),
            wildcard=dialog_format_string,
            defaultDir=str(window.file.find_default_dir()),
            defaultFile=str(window.file.path.stem),
            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
        )
        if export_dialog.ShowModal() == wx.ID_CANCEL:
            return

        # save our pathlib object
        file = pathlib.Path(export_dialog.GetPath())

        # use pandoc to convert
        try:
            # txt needs special handling...first to MD then through the filter
            if file.suffix == ".txt":
                file.write_text(
                    self.md_to_txt(
                        pypandoc.convert_text(
                            content,
                            format="markdown",
                            to="markdown",
                            outputfile=None,  # returning as a string in this case
                            extra_args=self.args
                        )
                    )
                )
            else:
                pypandoc.convert_text(
                    content,
                    format="markdown",
                    to=ext_to_pandoc_map[file.suffix],
                    outputfile=str(file),  # let pandoc write straight out to the file
                    extra_args=self.args
                )
        except Exception:  # NOQA
            wx.MessageBox(
                self._("Unable to export to the specified format."),
                self._("Error"),
                wx.OK | wx.ICON_ERROR
            )

        return

    @staticmethod
    def txt_to_md(text):
        return re.sub(r"(\r?\n)+", r"\n\n", text)  # normalize all line breaks to doubles

    @staticmethod
    def md_to_txt(text):
        return re.sub(r"(\r?\n)+", r"\n", text)  # normalize all line breaks to singles

    @staticmethod
    def normalize_line_breaks(content):
        content = re.sub(r"(\r?\n){2,}", "\n\n", content)
        content = re.sub(r"(\r?\n)+$", "", content)
        content = re.sub(r"(\\)", "", content)

        return content
