import pathlib          # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import re               # Python Regex - https://docs.python.org/3/library/re.html

import markdown2        # Markdown2 - https://github.com/trentm/python-markdown2
import wx               # wxPython - https://wxpython.org/


class FileHandler:
    def __init__(self, window):
        # get app obj
        self.app = wx.GetApp()
        self._ = self.app.constants.translation.gettext

        self.window = window
        self.path = pathlib.Path()
        self.lockfile = pathlib.Path()
        self.needs_save = False

        self.lockfile_suffix = ".~lock"

    def new(self):
        # handle unsaved changes (and cancel action if cancel was selected)
        if not self.handle_unsaved_changes():
            return

        # add closing file to list (if we have one) and delete lockfile
        if self.path.is_file():
            self.app.recents.file_list.insert(0, str(self.path.resolve()))
            if self.lockfile.is_file():
                self.lockfile.unlink()

        # clear everything
        self.window.text_committed_widget.SetValue("")
        self.window.text_working_widget.SetValue("")
        self.path = pathlib.Path()
        self.lockfile = pathlib.Path()
        self.needs_save = False
        self.app.recents.update(self.window)
        self.window.update_word_count(True)
        self.window.update_title()
        self.app.config.Write("files/current", "")
        self.app.config.Flush()

    # open and process a file
    def open(self, filename=None, is_import=False, raw_text=None):
        _ = self._

        # handle unsaved changes (and cancel action if cancel was selected)
        # on import action, we've already done this, so skip in that case
        if not is_import:
            if not self.handle_unsaved_changes():
                return

        # no filename was passed in; need to do the open dialog (unless we have raw text)
        if filename is None and raw_text is None:
            open_dialog = wx.FileDialog(
                self.window,
                _("Open a Commit document..."),
                wildcard="Commit Text Files (*.ctxt)|*.ctxt",
                defaultDir=str(self.find_default_dir()),
                style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
            )
            if open_dialog.ShowModal() == wx.ID_CANCEL:
                return
            else:
                filename = open_dialog.GetPath()

        # add closing file to list (if we have one) and remove lockfile
        if self.path.is_file():
            self.app.recents.file_list.insert(0, str(self.path))
            if self.lockfile.is_file():
                self.lockfile.unlink()

        # set current doc to newly opened one
        if filename is not None:
            file = pathlib.Path(filename)
            lockfile = pathlib.Path(
                str(file) + self.lockfile_suffix
            )
        else:
            file = pathlib.Path()
            lockfile = pathlib.Path()

        if raw_text is not None or file.is_file():
            if raw_text is not None:
                file_contents = raw_text
            else:
                # check if we're locked
                if not self.check_file_locked(lockfile):
                    return

                # load the file text
                file_contents = file.read_text()

            # put the text in the committed field to start
            self.window.text_committed_widget.SetValue(
                re.sub(r"(?<!\n)(\n)(?!\n)", r"\v", file_contents)
                # change any single line breaks to vertical tabs for display
            )

            # we start with an empty working field (will pull some text momentarily)
            self.window.text_working_widget.SetValue("")

            # do a pull (to populate the editing field)
            # this will also do a jump-to-bottom
            self.window.do_pull(on_open=True)

            # save the fileinfo and update the word count
            if is_import:
                self.path = pathlib.Path()
                self.lockfile = pathlib.Path()
                self.needs_save = True
            else:
                self.path = file
                self.lockfile = lockfile
                self.needs_save = False

                # create the lockfile!
                self.lockfile.touch()

            self.window.update_word_count(True)
            self.window.update_title()

            # set working widget insertion point at the end
            self.window.text_working_widget.SetInsertionPointEnd()

            # focus to the editing window
            self.window.text_working_widget.SetFocus()

            # save as current file in settings
            self.app.config.Write("files/current", str(self.path))

            # update the recents list
            self.app.recents.update(self.window)

            # save settings
            self.app.config.Flush()
        else:
            wx.MessageBox(
                _("Unable to open the specified file."),
                _("Error"),
                wx.OK | wx.ICON_ERROR
            )

    # save a file
    def save(self, save_as=False):
        _ = self._

        # prep the file obj
        file = self.path

        # we need to do a save-as, either because it's requested or because we need to
        if save_as or not file.is_file():
            save_dialog = wx.FileDialog(
                self.window,
                _("Save as..."),
                wildcard="Commit Text Files (*.ctxt)|*.ctxt",
                defaultDir=str(self.find_default_dir()),
                style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
            )
            if save_dialog.ShowModal() == wx.ID_CANCEL:
                return
            else:
                file = pathlib.Path(save_dialog.GetPath())

        # dist file content
        file_content = self.merge_content()

        # save to file
        file.write_text(
            file_content,
            encoding="utf-8",
            newline="\n"
        )  # change vertical tabs to single line breaks

        # save file info
        self.path = file

        # record that all is saved
        self.needs_save = False
        self.window.update_title()

    # print a file
    def print(self):
        printer = self.app.printer

        # set the name to filename (if we have one)
        if self.path.is_file():
            printer.SetName(self.path.name)
        else:
            printer.SetName(self.app.constants.app_display_name)

        # content
        raw_content = self.merge_content()
        parsed_content = markdown2.markdown(raw_content)

        # fire the print dialog
        printer.PrintText(parsed_content)

    # prompt the user to save before doing something destructive
    def check_file_locked(self, lockfile=pathlib.Path()):
        _ = self._

        if not lockfile.is_file():
            return True

        confirm_dialog = wx.MessageDialog(
            self.window,
            _("The selected file is locked and may be in use. Would you like to open it anyway?"),
            _("File Locked"),
            wx.YES_NO | wx.NO_DEFAULT | wx.ICON_WARNING
        )
        confirm_result = confirm_dialog.ShowModal()

        if confirm_result == wx.ID_NO:
            return False

        return True

    # prompt the user to save before doing something destructive
    def handle_unsaved_changes(self):
        _ = self._

        if not self.needs_save:
            return True

        confirm_dialog = wx.MessageDialog(
            self.window,
            _("Save changes before closing document?"),
            _("Confirm Action"),
            wx.YES_NO | wx.CANCEL | wx.YES_DEFAULT | wx.ICON_WARNING
        )
        confirm_result = confirm_dialog.ShowModal()

        if confirm_result == wx.ID_CANCEL:
            return False
        elif confirm_result == wx.ID_YES:
            self.save()
            return True

        return True

    # combine the content together (for save, print, export, etc.)
    def merge_content(self):
        committed_text = self.window.text_committed_widget.GetValue()
        working_text = self.window.text_working_widget.GetValue()

        if committed_text == "":
            combined_text = working_text
        else:
            combined_text = committed_text + "\n\n" + working_text

        # vertical tabs get saved, printed, and exported as single newlines
        return re.sub(r"\v", r"\n", combined_text)

    # default to current directory if we have one; otherwise, system defaults
    def find_default_dir(self):
        if self.path.is_file():
            # use current working directory if we have one
            return self.path.parent

        return None
