import re                   # Python Regex - https://docs.python.org/3/library/re.html

import wx               # wxPython - https://wxpython.org/


class SearchFindDialog():
    def __init__(self, window):
        self.dialog = wx.FindReplaceDialog(
            window,
            wx.FindReplaceData(
                wx.FR_DOWN
            ),
            "Find"
        )

        # get app obj
        self.app = wx.GetApp()
        app = self.app

        # init translations
        self._ = app.constants.translation.gettext

        self.window = window

        # some find variables
        self.find_pos = 0
        self.find_text = ["", ""]
        self.find_result = []
        self.find_settings = {}

        # bind the find dialog events
        self.dialog.Bind(wx.EVT_FIND, self.do_find)
        self.dialog.Bind(wx.EVT_FIND_NEXT, self.do_find)
        self.dialog.Bind(wx.EVT_FIND_CLOSE, self.close_find_dialog)

        # show the find dialog
        self.dialog.Show(True)

    def do_find(self, event):
        # init our vars from the event
        search_down = (event.GetFlags() & wx.FR_DOWN) != 0
        new_find_settings = {
            "find_string": event.GetFindString(),
            "whole_word": (event.GetFlags() & wx.FR_WHOLEWORD) != 0,
            "match_case": (event.GetFlags() & wx.FR_MATCHCASE) != 0
        }

        # clear selections
        self.window.text_committed_widget.SelectNone()
        self.window.text_working_widget.SelectNone()

        # test if this is the first search of this type (we actually do a one-time search and only iterate on find-next)
        # also test if the settings have changed, since this doesn't always trigger when it should (sigh...)
        # 10155 = wx.EVT_FIND; 10156 = wx.EVT_FIND_NEXT
        if event.GetEventType() == 10155 or new_find_settings != self.find_settings:
            self.find_settings = new_find_settings

            # reset to 0 position
            self.find_pos = -1

            # clear previous result matrix
            self.find_result = []

            # get our text (only on first search)
            self.find_text[0] = self.window.text_committed_widget.GetValue()
            self.find_text[1] = self.window.text_working_widget.GetValue()

            # prepare the regex

            # first, escape the input
            re_string = re.escape(self.find_settings["find_string"])

            # if it's a "whole word" search
            if self.find_settings["whole_word"]:
                re_string = "\\b" + re_string + "\\b"

            # handle the match-case flag
            if self.find_settings["match_case"]:
                re_flags = 0
            else:
                re_flags = re.IGNORECASE

            # compile it all to a regex object
            re_obj = re.compile(re_string, re_flags)

            # do the search and build out the result array
            for idx, this_text in enumerate(self.find_text):
                for this_match in re_obj.finditer(this_text):
                    self.find_result.append([idx, this_match.start(), this_match.end()])

        if len(self.find_result) > 0:
            # if we have results

            # update the index pointer
            if search_down:
                self.find_pos += 1
            else:
                self.find_pos -= 1

            # handle if we looped around
            if self.find_pos >= len(self.find_result) - 1:
                wx.Bell()
                self.find_pos = 0
            elif self.find_pos < 0:
                if self.find_pos >= -1:
                    wx.Bell()
                self.find_pos = len(self.find_result) - 1

            # determine target field
            if self.find_result[self.find_pos][0] == 0:
                target = self.window.text_committed_widget
            else:
                target = self.window.text_working_widget

            # highlight the text and make sure it's visible
            target_start = self.find_result[self.find_pos][1]
            target_end = self.find_result[self.find_pos][2]
            target.ShowPosition(target_start)
            target.SetSelection(target_start, target_end)

        else:
            # no results found
            wx.Bell()

        # set statusbar text
        self.window.status_bar.SetStatusText(
            str(self.find_pos + 1) + " " + self._("of") + " " + str(len(self.find_result)) + " " + self._("results found"), 0
        )  # NOQA

    # we need a custom close handler (or else the titlebar close button doesn't work on Mac and Linux...reasons!)
    # we also use this to clear out the now-unneeded search variables
    def close_find_dialog(self, inner_event):
        inner_event.StopPropagation()

        # reset everything
        self.find_text = ["", ""]
        self.find_result = []
        self.find_pos = [0]

        # restore the statusbar
        self.window.update_word_count()

        # destroy!
        self.dialog.Destroy()
        inner_event.Skip()
