import locale               # Python Locale - https://docs.python.org/3/library/locale.html
import pathlib              # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import re                   # Python Regex - https://docs.python.org/3/library/re.html
import webbrowser           # Python WebBrowser - https://docs.python.org/3/library/webbrowser.html

import wx                   # wxPython - https://wxpython.org/
from wx.adv import Sound    # wxPython Sound - https://docs.wxpython.org/wx.adv.Sound.html

from commit.includes.filehandler import FileHandler       # local - file open, save, and path tools
from commit.includes.fileconvert import FileConvert       # local - file conversion (import/export)
from commit.includes.searchfind import SearchFindDialog   # local - searching and finding
from commit.ui.commit_ui_design import MainWin_Base       # local - main window design from wxFormBuilder
from commit.ui.aboutwin import AboutWin                   # local - about window
from commit.ui.prefswin import PrefsWin                   # local - preferences window


class MainWin(MainWin_Base):
    def __init__(self, parent, start_file):
        # call parent init
        super().__init__(parent)

        # init must-have values to their defaults
        self.word_count_start = 0
        self.scroll_bypass = False

        # get app obj
        app = wx.GetApp()
        self.app = app

        # why is locale stuff so complicated?
        if self.app.constants.platform in ["Windows", "Darwin"]:
            locale.setlocale(
                locale.LC_ALL,
                locale.getdefaultlocale()[0]
            )
        else:
            locale.setlocale(
                locale.LC_ALL,
                ""
            )

        # init translations
        self._ = app.constants.translation.gettext
        _ = self._

        # file handle object
        self.file = FileHandler(self)

        # import/export object
        self.convert = FileConvert()

        # set text field margins
        field_margins = wx.Point(10, -1)
        self.text_committed_widget.SetMargins(field_margins)
        self.text_working_widget.SetMargins(field_margins)

        # set window icon
        self.SetIcons(app.constants.icon)

        # should fullscreen?
        # need to figure it out here, but we actually do it later (has to be after window show)
        should_fullscreen = app.config.ReadBool("window/is_fullscreen")

        # should maximize?
        if app.config.ReadBool("window/is_maximized"):
            self.Maximize(True)

        # get the screen size
        window_size = wx.DisplaySize()

        # determine window size (previous, unless that's weird)
        new_size = [
            app.config.ReadInt("window/size_x"),
            app.config.ReadInt("window/size_y")
        ]
        if new_size[0] > window_size[0]:
            new_size[0] = round(window_size[0] * 0.9)
        if new_size[1] > window_size[1]:
            new_size[1] = round(window_size[1] * 0.9)

        # determine window position (previous, unless that's weird)
        new_pos = [
            app.config.ReadInt("window/pos_x"),
            app.config.ReadInt("window/pos_y")
        ]
        if new_pos[0] > window_size[0] or new_pos[0] < 0:
            new_pos[0] = -1
        if new_pos[1] > window_size[1] or new_pos[1] < 0:
            new_pos[1] = -1

        # dooo it!
        self.SetSize(
            new_pos[0],
            new_pos[1],
            new_size[0],
            new_size[1]
        )

        # center if the new position is default values
        if new_pos[0] == -1 or new_pos[1] == -1:
            self.Center()

        # handle our custom icons; first some cleanup
        icon_size = wx.Size(24, 24)

        # first, up-arrow
        commit_icon = wx.ArtProvider.GetBitmap(wx.ART_GO_UP, wx.ART_TOOLBAR, icon_size)

        # next, down-arrow
        pull_icon = wx.ArtProvider.GetBitmap(wx.ART_GO_DOWN, wx.ART_TOOLBAR, icon_size)

        # handle the settings icon; it is missing from the art provider (WTF?)
        setting_icon = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE, wx.ART_TOOLBAR, icon_size)

        # in case it ever gets added by the art provider
        try:
            setting_icon = wx.ArtProvider.GetBitmap(wx.ART_PREFERENCES, wx.ART_TOOLBAR, icon_size)  # noqa
        except AttributeError:
            pass

        # now try the GTK version
        setting_icon_gtk = wx.ArtProvider.GetBitmap("gtk-preferences", wx.ART_TOOLBAR, icon_size)
        if setting_icon_gtk.IsOk():
            setting_icon = setting_icon_gtk

        # fullscreen is missing on some platforms (WHY SO ANNOYING?); start with missing image icon
        self.fullscreen_icon = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE, wx.ART_TOOLBAR, icon_size)
        self.fullscreen_icon_back = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE, wx.ART_TOOLBAR, icon_size)

        # now set to the ART_FULL_SCREEN icon, if it exists
        try:
            self.fullscreen_icon = wx.ArtProvider.GetBitmap(wx.ART_FULL_SCREEN, wx.ART_TOOLBAR, icon_size)
            self.fullscreen_icon_back = wx.ArtProvider.GetBitmap(wx.ART_FULL_SCREEN, wx.ART_TOOLBAR, icon_size)
        except AttributeError:
            pass

        # now set to the gtk icons, if they're available
        fullscreen_icon_gtk = wx.ArtProvider.GetBitmap("gtk-fullscreen", wx.ART_TOOLBAR, icon_size)
        fullscreen_icon_back_gtk = wx.ArtProvider.GetBitmap("gtk-leave-fullscreen", wx.ART_TOOLBAR, icon_size)
        if fullscreen_icon_gtk.IsOk() and fullscreen_icon_back_gtk.IsOk():
            self.fullscreen_icon = fullscreen_icon_gtk
            self.fullscreen_icon_back = fullscreen_icon_back_gtk

        # and finally, the exit button
        quit_icon = wx.ArtProvider.GetBitmap(wx.ART_QUIT, wx.ART_TOOLBAR, icon_size)

        # override icons for mac
        if app.constants.platform == "Darwin":
            commit_icon = wx.Bitmap(
                "commit/assets/toolbar/mac/chevron-up-24.png",
                wx.BITMAP_TYPE_PNG
            )
            pull_icon = wx.Bitmap(
                "commit/assets/toolbar/mac/chevron-down-24.png",
                wx.BITMAP_TYPE_PNG
            )
            setting_icon = wx.Bitmap(
                "commit/assets/toolbar/mac/gear-24.png",
                wx.BITMAP_TYPE_PNG
            )
            self.fullscreen_icon = wx.Bitmap(
                "commit/assets/toolbar/mac/full-screen-24.png",
                wx.BITMAP_TYPE_PNG
            )
            self.fullscreen_icon_back = wx.Bitmap(
                "commit/assets/toolbar/mac/full-screen-back-24.png",
                wx.BITMAP_TYPE_PNG
            )
            quit_icon = wx.Bitmap(
                "commit/assets/toolbar/mac/escape-24.png",
                wx.BITMAP_TYPE_PNG
            )

        # override icons for windows
        if app.constants.platform == "Windows":
            commit_icon = wx.Bitmap(
                "commit/assets/toolbar/windows/commit-24.png",
                wx.BITMAP_TYPE_PNG
            )
            pull_icon = wx.Bitmap(
                "commit/assets/toolbar/windows/pull-24.png",
                wx.BITMAP_TYPE_PNG
            )
            setting_icon = wx.Bitmap(
                "commit/assets/toolbar/windows/prefs-24.png",
                wx.BITMAP_TYPE_PNG
            )
            self.fullscreen_icon = wx.Bitmap(
                "commit/assets/toolbar/windows/fullscreen-24.png",
                wx.BITMAP_TYPE_PNG
            )
            self.fullscreen_icon_back = wx.Bitmap(
                "commit/assets/toolbar/windows/fullscreen-back-24.png",
                wx.BITMAP_TYPE_PNG
            )
            quit_icon = wx.Bitmap(
                "commit/assets/toolbar/windows/quit-24.png",
                wx.BITMAP_TYPE_PNG
            )

        # now set all the values
        self.LeftToolbar.SetToolNormalBitmap(
            self.ToolCommit.GetId(),
            commit_icon
        )
        self.LeftToolbar.SetToolNormalBitmap(
            self.ToolPull.GetId(),
            pull_icon
        )
        self.RightToolbar.SetToolNormalBitmap(
            self.ToolPrefs.GetId(),
            setting_icon
        )
        self.RightToolbar.SetToolNormalBitmap(
            self.ToolFullScreen.GetId(),
            self.fullscreen_icon
        )
        self.RightToolbar.SetToolNormalBitmap(
            self.ToolExit.GetId(),
            quit_icon
        )

        # by default, remove the "quit" toolbar icon (only show in fullscreen mode)
        self.RightToolbar.RemoveTool(
            self.ToolExit.GetId()
        )

        self.LeftToolbar.SetToolBitmapSize(self.FromDIP(wx.Size(24, 24)))
        self.RightToolbar.SetToolBitmapSize(self.FromDIP(wx.Size(24, 24)))

        # realize!
        self.LeftToolbar.Realize()
        self.RightToolbar.Realize()

        # bind scroll events (so we can prevent them if scrolling committed window is disabled)
        self.text_committed_widget.Bind(wx.EVT_SCROLLWIN, self.event_committed_scroll)
        self.text_committed_widget.Bind(wx.EVT_MOUSEWHEEL, self.event_committed_scroll)

        # bind the paste event on the working text field
        self.text_working_widget.Bind(wx.EVT_TEXT_PASTE, self.event_working_paste)

        # set prefs!
        self.do_prefs_update()

        # enable Apple macOS fullscreen capability (if the method exists, since it's newish)
        if hasattr(self, "EnableFullScreenView") and callable(self.EnableFullScreenView):
            self.EnableFullScreenView(True)
            self.Bind(wx.EVT_FULLSCREEN, self.do_toggle_fullscreen)

        # show the window!
        self.Show()

        # focus to the editing window
        self.text_working_widget.SetFocus()

        # should fullscreen? has to be after show
        # we determine it earlier though, so it doesn't get munged by window sizing
        if should_fullscreen:
            self.do_toggle_fullscreen(force=True)

        # save the last opened file to a var
        last_file = pathlib.Path(app.config.Read("files/current"))

        if start_file.is_file() and start_file.suffix == ".ctxt":
            # if a file was passed in on the command line; save last to recents and open req file
            app.recents.file_list.insert(0, str(last_file.resolve()))
            self.file.open(start_file.resolve())
        elif app.config.ReadBool("startup/open_last_edited"):
            # else if we have a setting to open last file, and the file is valid
            file = pathlib.Path(app.config.Read("files/current"))
            if file.is_file():
                self.file.open(file.resolve())
            else:
                self.update_word_count(True)
                self.app.recents.update(self)
        elif app.config.Read("files/current") != "":
            # if we're not opening on startup, last open file to recent files list and remove from current
            app.recents.file_list.insert(0, str(last_file.resolve()))
            app.config.Write("files/current", "")

        # update word count and recents
        self.update_word_count(True)
        self.app.recents.update(self)

    # provide FromDIP method if we don't already have one (i.e., support older wxPython)
    def FromDIP(self, size: wx.Size = None):
        if hasattr(super(), "EnableFullScreenView") and callable(super().FromDIP):
            return super().FromDIP(size)
        else:
            return size

    # do a commit
    def do_commit(self, consider_position=False):
        # get the info we need
        selection = self.text_working_widget.GetSelection()
        end_point = self.text_working_widget.GetInsertionPoint()
        content_length = len(self.text_working_widget.GetValue())

        # we don't *really* want to commit if there's no content
        if content_length == 0:
            return

        if self.app.config.ReadBool("options/commit-sounds"):
            Sound().PlaySound("commit/assets/sound/commit.wav")

        if selection[0] != selection[1]:
            # if we have a selection, that's what we're committing
            # doesn't matter if we pressed "enter" or used a button/menu
            range_to_commit = [selection[0], selection[1]]
        elif end_point > 0 and consider_position:
            # if considering position (i.e., comes from an enter press and not at pos 0)
            # here we commit from 0 to the insertion point
            range_to_commit = [0, end_point]
        else:
            # if we're in position 0, or if we came from a button/menu, but with no selection
            # in this case, we commit the whole working field
            range_to_commit = [0, self.text_working_widget.GetLastPosition()]

        # append the text we're committing
        if self.text_committed_widget.GetValue() != "":
            self.text_committed_widget.AppendText("\n\n")
        self.text_committed_widget.AppendText(
            (self.text_working_widget.GetRange(
                range_to_commit[0],
                range_to_commit[1]
            )).strip()
        )

        # remove the committed text from the working field
        self.text_working_widget.Remove(range_to_commit[0], range_to_commit[1])

        # scroll to bottom
        self.do_jump_to_bottom()

        # set that we need an update
        self.file.needs_save = True

        # update the committed word count
        self.update_word_count()

        # save the file (if we have autosave enabled AND a filename)
        if self.file.path.is_file() and self.app.config.ReadBool("options/auto-save-commit"):
            self.file.save()
        else:
            self.file.needs_save = True
            self.update_title()

    # do a pull
    def do_pull(self, on_open=False):
        committed_content = self.text_committed_widget.GetValue()
        working_content = self.text_working_widget.GetValue()

        # can't pull if nothing is there!
        if committed_content == "":
            return

        # start with nothing
        pull_text = None

        # find the last paragraph
        re_obj = re.compile(r"(\n\n)(.*?)$")

        # gather the needed information
        pull_result = re_obj.search(committed_content)

        if pull_result is not None and pull_result.group(2) is not None:
            pull_text = pull_result.group(2)
        else:
            re_obj = re.compile(r"^(.*)$")
            pull_result = re_obj.search(committed_content)
            if pull_result is not None and pull_result.group(1) is not None:
                pull_text = pull_result.group(1)

        if pull_result is None or pull_text is None:
            return

        if not on_open and self.app.config.ReadBool("options/commit-sounds"):
            Sound().PlaySound("commit/assets/sound/pull.wav")

        # first, delete the content we're pulling
        self.text_committed_widget.Remove(pull_result.start(), pull_result.end())

        if working_content is not None and working_content != "":
            # if we have working content, we need to prepend
            self.text_working_widget.SetValue(pull_text + " " + working_content)
        else:
            # the working field is empty; just plop the text in it!
            self.text_working_widget.SetValue(pull_text)

        # scroll to ends
        self.do_jump_to_bottom()

        # update the committed word count
        self.update_word_count()

        # and mark that we need a save, unless this happened on a new open
        if not on_open:
            self.file.needs_save = True
            self.update_title()

    # jump to the bottom of the committed text field
    def do_jump_to_bottom(self):
        committed = self.text_committed_widget

        self.scroll_bypass = True

        # freeze (reduce flicker)
        self.Freeze()

        # set committed widget insertion point at the end
        committed.SetInsertionPoint(
            committed.GetLastPosition()
        )
        committed.ShowPosition(
            committed.GetLastPosition()
        )

        # workaround problem where it's not always REALLY at the bottom (pgdown)
        committed.ScrollPages(1)

        # update
        self.scroll_bypass = False

        # thaw (update visual)
        self.Thaw()

    # preferences have been updated; refresh some things
    def do_prefs_update(self):
        # when prefs are saved, update word count
        self.update_word_count()

        # update font size
        font = self.text_working_widget.GetFont()
        font.SetPointSize(self.app.config.ReadInt("view/font_size"))
        self.text_working_widget.SetFont(font)
        self.text_committed_widget.SetFont(font)

    # toggle fullscreen status
    def do_toggle_fullscreen(self, event=None, force=False):
        self.Show()  # ugh; silly buggy bug bug!
        should_fullscreen = (not self.IsFullScreen() or force)

        if event is not None:
            # if we got an event, it means this was triggered by the Mac green button
            # this means we need to get the fullscreen status a different way (reasons??)
            # it also does its own toggle, so we skip the toggle step (only in else)
            should_fullscreen = event.IsFullScreen()
        else:
            # do the toggle, unless macOS is already doing it
            self.ShowFullScreen(
                should_fullscreen,
                wx.FULLSCREEN_NOBORDER | wx.FULLSCREEN_NOMENUBAR
            )

        if should_fullscreen:
            self.RightToolbar.AddTool(self.ToolExit)
            self.ToolExit.SetLabel(self._("Quit"))  # WHY do we have to reset this here!?
            self.RightToolbar.SetToolNormalBitmap(
                self.ToolFullScreen.GetId(),
                self.fullscreen_icon_back
            )
        else:
            self.RightToolbar.RemoveTool(self.ToolExit.GetId())
            self.RightToolbar.SetToolNormalBitmap(
                self.ToolFullScreen.GetId(),
                self.fullscreen_icon
            )

        # mindscape realizer
        self.RightToolbar.Realize()

        # save fullscreen status
        self.app.config.WriteBool("window/is_fullscreen", should_fullscreen)

    # update the title and saved indicator
    def update_title(self):
        new_title = self.app.constants.app_display_name

        if self.file.path.is_file():
            new_title = self.file.path.name + " – " + new_title

        if self.file.needs_save:
            if self.app.constants.platform != "Darwin":
                # only put the dot in the title on non-Mac platforms
                new_title = "• " + new_title
            # handle the modification icon on Mac
            self.OSXSetModified(True)
        else:
            self.OSXSetModified(False)

        self.SetTitle(new_title)

    # calculate and show the word count
    def update_word_count(self, reset_doc_count=False):
        this_length = len(self.text_committed_widget.GetValue().split())

        if reset_doc_count:
            self.word_count_start = this_length

        if self.app.config.ReadBool("statusbar/show_word_count"):
            format_length = locale.format_string("%d", this_length, grouping=True, monetary=True)

            # insert into statusbar
            if this_length == 1:
                self.status_bar.SetStatusText(format_length + " " + self._("committed word"), 0)
            else:
                self.status_bar.SetStatusText(format_length + " " + self._("committed words"), 0)
        else:
            self.status_bar.SetStatusText("", 0)

        if self.app.config.ReadBool("statusbar/show_session_word_count"):
            session_words = this_length - self.word_count_start
            if session_words < 0:
                session_words = 0

            format_length = locale.format_string("%d", session_words, grouping=True)

            # insert into statusbar
            if session_words == 1:
                self.status_bar.SetStatusText(format_length + " " + self._("new word in this session"), 1)
            else:
                self.status_bar.SetStatusText(format_length + " " + self._("new words in this session"), 1)
        else:
            self.status_bar.SetStatusText("", 1)

    # when the app closes
    def event_app_close(self, event):
        if self.file.handle_unsaved_changes():
            if self.file.lockfile.is_file():
                self.file.lockfile.unlink()
            event.Skip()

    def event_window_resize(self, event):
        if event.GetEventType() == 10086:
            if self.app.config.ReadBool("options/disable-scrolling"):
                # if scrolling is disabled, always jump to bottom
                self.do_jump_to_bottom()

        # always do the normal action
        event.Skip()

        # config object
        config = self.app.config

        # get size and position
        window_size = self.GetSize().Get()
        window_pos = self.GetPosition().Get()

        # now save it all back to the prefs
        config.WriteBool("window/is_maximized", self.IsMaximized())
        config.WriteBool("window/is_fullscreen", self.IsFullScreen())

        # but don't write the size and pos
        if not self.IsMaximized() and not self.IsFullScreen():
            config.WriteInt("window/size_x", window_size[0])
            config.WriteInt("window/size_y", window_size[1])
            config.WriteInt("window/pos_x", window_pos[0])
            config.WriteInt("window/pos_y", window_pos[1])

    # fires on any keydown event inside the editing window
    def event_working_key_press(self, event):
        current_insertion_point = event.GetEventObject().GetInsertionPoint()

        # detect commit/pull scenarios (enter)
        if (
            event.GetModifiers() != wx.MOD_SHIFT and
            event.GetKeyCode() in [wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER]
        ):
            # commit unless we're holding the shift key
            self.do_commit(True)
        elif (
            event.GetKeyCode() == wx.WXK_BACK and
            current_insertion_point == 0 and
            self.text_working_widget.GetStringSelection() == ""
        ):
            self.do_pull()
        else:
            # in any other case, keypress should do whatever it normally does
            event.Skip()

    def event_committed_key_press(self, event):
        if (
            self.app.config.ReadBool("options/disable-scrolling") and
            event.GetKeyCode() in [
                wx.WXK_PAGEDOWN,
                wx.WXK_PAGEUP,
                wx.WXK_UP,
                wx.WXK_DOWN,
                wx.WXK_LEFT,
                wx.WXK_RIGHT,
                wx.WXK_HOME,
                wx.WXK_END
            ]
        ):
            event.StopPropagation()
        elif event.GetKeyCode() == wx.WXK_END:
            self.do_jump_to_bottom()
        elif event.GetKeyCode() == wx.WXK_HOME:
            self.text_committed_widget.ShowPosition(0)
        else:
            event.Skip()

    # when we scroll in the committed window, we sometimes want to stop it (there's a setting!)
    def event_committed_scroll(self, event):
        if not self.scroll_bypass and self.app.config.ReadBool("options/disable-scrolling"):
            event.StopPropagation()
        else:
            event.Skip()

    # whenever the text is changed
    def event_working_text_changed(self, event):
        self.file.needs_save = True
        self.update_title()
        event.Skip()

    # when something is pasted
    def event_working_paste(self, event):
        # only allow pasting plain text!
        clipboard = wx.Clipboard()

        # open clipboard
        if (
            (
                wx.TheClipboard.IsSupported(wx.DataFormat(wx.DF_UNICODETEXT)) or
                wx.TheClipboard.IsSupported(wx.DataFormat(wx.DF_TEXT))
            ) and
            clipboard.Open()
        ):
            # get clipboard data
            data = wx.TextDataObject()
            clipboard.GetData(data)
            clipboard.Close()

            # insert text into field
            self.text_working_widget.WriteText(
                data.GetText()
            )
        else:
            # nothing valid to paste
            wx.Bell()

    def event_menu_file_new(self, event):
        self.file.new()

    def event_menu_file_open(self, event):
        self.file.open()

    def event_menu_file_save(self, event):
        self.file.save()

    def event_menu_file_save_as(self, event):
        self.file.save(save_as=True)

    # open recent item from menu
    def event_menu_file_recent_open(self, event):
        real_id = event.GetId() - 10000
        self.file.open(
            self.app.recents.file_list[real_id]
        )

    # reset recent items menu
    def event_menu_file_recent_reset(self, event):
        self.app.recents.reset(self)

    def event_menu_file_prefs(self, event):
        # pop open the "prefs" window
        prefs_win = PrefsWin(self)
        pref_result = prefs_win.ShowModal()

        if pref_result == wx.ID_SAVE:
            self.do_prefs_update()

    def event_menu_file_import(self, event):
        if not self.file.handle_unsaved_changes():
            return

        if not self.convert.pandoc_check():
            return

        # hand off to the importer
        self.convert.do_import(self)

    def event_menu_file_export(self, event):
        if not self.file.handle_unsaved_changes():
            return

        if not self.convert.pandoc_check():
            return

        # hand off to the exporter
        self.convert.do_export(self, self.file.merge_content())

    def event_menu_file_pagesetup(self, event):
        self.app.printer.PageSetup()

    def event_menu_file_print(self, event):
        self.file.print()

    def event_menu_file_exit(self, event):
        self.Close()
        self.Destroy()

    def event_menu_edit_undo(self, event):
        # perform an undo (only in the working area)
        self.text_working_widget.Undo()

    def event_menu_edit_redo(self, event):
        # perform a redo (only in the working area)
        # FIXME this should work...but it doesn't :-/
        self.text_working_widget.Redo()

    def event_menu_edit_cut(self, event):
        # Cut from the focused area (if allowed)
        self.FindFocus().Cut()

    def event_menu_edit_copy(self, event):
        # Copy from the focused area (if allowed)
        self.FindFocus().Copy()

    def event_menu_edit_paste(self, event):
        # Paste into the focused area (if allowed)
        self.FindFocus().Paste()

    def event_menu_edit_delete(self, event):
        # Delete the selected text in the working area (if any)
        my_selection = self.text_working_widget.GetSelection()
        self.text_working_widget.Remove(my_selection[0], my_selection[1])

    def event_menu_select_all(self, event):
        # Select all text in the focused area
        self.FindFocus().SelectAll()

    def event_menu_edit_commit(self, event):
        self.do_commit()

    def event_menu_edit_pull(self, event):
        self.do_pull()

    def event_menu_edit_find(self, event):
        SearchFindDialog(self)

    def event_menu_window_minimize(self, event):
        # Minimize the window (which is called "iconize" for some reason)
        self.Iconize(True)

    def event_menu_window_maximize(self, event):
        _ = self._

        # Toggle maximize status
        self.Maximize(
            not self.IsMaximized()
        )

        # what should the menu label be?
        if self.IsMaximized():
            menu_label = _("Restore")
        else:
            menu_label = _("Maximize")

        # set menu label
        self.menu_maximize.SetItemLabel(menu_label)

    def event_menu_window_fullscreen(self, event):
        self.do_toggle_fullscreen()

    def event_menu_help_website(self, event):
        # open the app website
        webbrowser.open(self.app.constants.app_website)

    def event_menu_help_manual(self, event):
        # open the help/manual website
        webbrowser.open(self.app.constants.app_help_url)

    def event_menu_help_updates(self, event):
        # fire an update
        self.app.updater.do_update_check()

    def event_menu_help_about(self, event):
        # pop open the "about" window
        about_win = AboutWin(self)
        about_win.ShowWindowModal()
